:
# RCSid:
#	$Id: sb-funcs.sh,v 1.28 2020/12/05 17:13:04 sjg Exp $
#
#	@(#) Copyright (c) 2009-2020 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

Myname=${Myname:-`basename $0 .sh`}
Mydir=${Mydir:-`dirname $0`}
pwd=${pwd:-'pwd'}
case "$Mydir" in
.) Mydir=`$pwd`;;
esac
SB_TOOLS=${SB_TOOLS:-$Mydir}
$_OS_SH . $SB_TOOLS/os.sh
$_HOOKS_SH . $SB_TOOLS/hooks.sh
$_ADD_PATH_SH . $SB_TOOLS/add_path.sh
$_TEST_OPTS_SH . $SB_TOOLS/test_opt.sh

ev=.sandbox-env
rc=.sandboxrc

# turn something into a useful var name
varName() {
    case $1 in
    *[!A-Za-z0-9_]*) vn=`echo $1 | sed 's,[^A-Za-z0-9_],_,g'`;;
    *) vn=$1;;
    esac
    echo $vn
}

evalVar() {
    val="$@"
    while :
    do
        case "$val" in
	*\$*) eval "val=\"$val\"";;
	*) break;;
	esac
    done
    echo "$val"
}
    
# we sometimes need these as var names
# MYNAME is our canonical name
# Myname is from $0
for n in Myname MYNAME
do
    eval vn=\$$n
    vn=`varName $vn`
    eval var${n}=$vn
done

# find_it [--start $dir][-$test][--{path,dir}] $path ...
# find the first $path in $start or above
# if --path report $dir/$path otherwise just $dir where $path found
find_it() {
    _t=-s
    _start=.
    _what=dir
    while :
    do
	case "$1" in
	-?) _t=$1; shift;;
	--path) _what=path; shift;;
	--dir) _what=dir; shift;;
	--start) _start=$2; shift; shift;;
	*) break;;
	esac
    done
    cd $_start > /dev/null 2>&1 || return 1
    while :
    do
	here=`$pwd`
	for it in "$@"
	do
	    if test $_t $it; then
		case "$_what" in
		path) echo $here/$it;;
		*) echo $here;;
		esac
		return
	    fi
	done
	case $here in
	/) return;;
	esac
	cd ..
    done
}

find_sb() {
    find_it --start ${1:-.} --dir $ev
    
}

error() {
    echo "ERROR: $@" >&2
    exit 1
}

source_rc() {
    local f source_dir source_file

    for f in "$@"
    do
	[ -s $f ] || continue
	case $f in
	*/*) source_dir=${f%/*};;
	*) source_dir=.;;
	esac
	source_file=${f##*/}
	. $f
    done
}

sb_hooks() {
    cd $1 || exit 1
    SB=`$pwd`
    SB_NAME=`basename $SB`
    export SB SB_NAME

    source_rc ../$rc ./$rc
    run_hooks sb_init_hooks
    run_hooks ${varMYNAME}_init_hooks
    source_rc ./$ev
    run_hooks sb_setup_hooks
    run_hooks ${varMYNAME}_setup_hooks
}

add_list() {
    _list=$1; shift
    eval "$_list=\"\$$_list $@\""
}

sort_list() {
    case "$1" in
    --) _u=; shift;;
    -[run]*) _u=$1; shift;;
    *) _u=;;
    esac
    case "$1" in
    "") return;;
    esac
    for i in "$@"
    do
        echo $i
    done | sort $_u
}

# we want to re-use this
extract_doc() {
    case "$1" in
    eod=*) eval "$1"; shift;;
    *) eod=${END_HELP_MARKER:-RCSid};;
    esac
    for f in "$@"
    do
        sed -n -e "1d;/$eod/,\$d" -e '/^#\.[a-z]/d' -e '/^#/s,^# *,,p' $f
    done
}
        
# an easy way to add --help support to shell scripts
# even those that use getopt(1) is (before that) do:
# # Yes I mean $* below, and the spaces.
# case " $* " in *" --help "*) __help;; esac
__doc() {
    extract_doc ${1:-$0}
    run_hooks doc_hooks
    exit 0
}

__help() {
    case "$1" in
    ""|help) topic=;;
    _*) topic=$1;;
    *) topic=_$1;;
    esac
    nl=
    for n in $varMyname $varMYNAME $MYNAME
    do
	case " ${nl} " in
	*" $n "*) continue;;
	esac
	nl="$nl $n"

	# look for hooks
	for hl in ${n}_help${topic}_hooks ${n}${topic:-_help}_hooks help${topic}_hooks
	do
	    eval hh=\$$hl
	    if [ -n "$hh" ]; then
		run_hooks $hl
		exit 0
	    fi
	done
	# look for function
	for hf in ${n}_help${topic} help${topic}
	do
	    # type is available in most shell's post 1980
	    # users who alias ${n}_help to something
	    # that does not exist, deserve to lose.
	    : hf=$hf
	    case `(type $hf) > /dev/null 2>&1` in
	    *function*)
		$hf
		exit 0
		;;
	    esac
	done
    done
    case "$topic" in
    "") __doc;;
    *) error no help$topic found;;
    esac
    exit 0
}
